@@ -1,8 +1,7 @@
 #!/bin/sh
 # script to set MCPP to be called from gcc
-# ./set_mcpp.sh $gcc_path $gcc_maj_ver $gcc_min_ver $cpp_call $CC   \
-#       $CXX x$CPPFLAGS x$EXEEXT $LN_S $inc_dir $host_system $cpu $cpu32
-#       $target_cc
+# ./set_mcpp.sh $gcc_path $gcc_maj_ver $gcc_min_ver $cpp_call $CC
+#       $CXX x$CPPFLAGS x$EXEEXT $LN_S $inc_dir $host_system $cpu $target_cc
 
 gcc_maj_ver=$2
 gcc_min_ver=$3
@@ -14,9 +13,8 @@ LN_S=$9
 inc_dir=${10}
 host_system=${11}
 cpu=${12}
-cpu32=${13}
 if test $host_system = SYS_MAC; then
-    target_cc=${14}
+    target_cc=${13}
     target=`echo $target_cc | sed 's/-gcc.*$//'`
 fi
 cpp_name=`echo $cpp_call | sed 's,.*/,,'`
@@ -36,6 +34,12 @@ if test $host_system = SYS_MINGW && test ! -f cc1$EXEEXT; then
     echo "  first do 'make COMPILER=GNUC mcpp cc1'; then do 'make COMPILER=GNUC install'"
     exit 1
 fi
+if test x$cpp_base = xcc1; then
+    # for GCC V.3.3 and later
+    no_m64=0
+else
+    no_m64=1
+fi
 
 gen_headers() {
     echo "  mkdir -p $hdir"
@@ -61,40 +65,42 @@ cwd=`pwd`
 echo "  cd $inc_dir"
 cd $inc_dir
 
-if test $host_system = SYS_MAC; then
-## Apple-GCC changes architecture and predefined macros by -arch * option
-    if test $cpu = i386 || test $cpu = x86_64; then
-        cpu=x86_64
-        cpu32=i386
-    else 
-        if test $cpu = ppc || $cpu = ppc64; then
-            cpu=ppc64
-            cpu32=ppc
-        fi
+if test $cpu = i386 || test $cpu = x86_64; then
+    cpu32=i386
+    cpu64=x86_64
+else 
+    if test $cpu = ppc || $cpu = ppc64; then
+        cpu32=ppc
+        cpu64=ppc64
     fi
 fi
 
 arch_headers() {
-    for arch in $cpu $cpu32
+    for arch in $cpu32 $cpu64
     do                              ## generate headers for 2 architectures
         hdir=${idir}-$arch
         if test $host_system = SYS_MAC; then
             arg="-arch $arch"
         else
-            if test $cpu = x86_64 || test $cpu = ppc64; then
-                if test $arch = $cpu; then
-                    arg="$ar -m64"
-                else
-                    arg="$ar -m32"
-                fi
+            if test $arch = $cpu; then
+                arg="$ar"
             else
-                arg=
+                if test $host_system = SYS_MINGW || test $no_m64; then
+                    continue;
+                fi
+                if test $cpu = $cpu64; then
+                    arg="$ar -m32"
+                else
+                    arg="$ar -m64"
+                fi
+                # Test if the architecture is supported.
+                $CC -E -xc $arg /dev/null > /dev/null
+                if test $? != 0; then
+                    continue
+                fi
             fi
         fi
         gen_headers
-        if test $cpu32 = $cpu; then
-            break;
-        fi
     done
 }
 
@@ -116,6 +122,7 @@ cd $cpp_path
 
 # other than MinGW
 if test $host_system != SYS_MINGW; then
+    echo '#!/bin/sh'                    >  mcpp.sh
     # for GCC V.3.3 and later
     if test x$cpp_base = xcc1; then
         for cpp in cc1 cc1plus
@@ -125,8 +132,10 @@ if test $host_system != SYS_MINGW; then
             else
                 shname=mcpp_plus
             fi
-            cat > $shname.sh <<_EOF
-#!/bin/sh
+            if test $cpp = cc1plus; then
+                echo '#!/bin/sh'        >  mcpp_plus.sh
+            fi
+            cat >> $shname.sh <<_EOF
 for i in \$@
 do
     case \$i in
@@ -144,10 +153,10 @@ _EOF
     if test $host_system = SYS_MAC && test -f ${target}-mcpp; then
         mcpp_name=${target}-mcpp    ## long name of Mac OS X cross-compiler
     fi
-    echo $cpp_path/$mcpp_name '"$@"'   >>  mcpp.sh
+    echo $cpp_path/$mcpp_name '"$@"'    >>  mcpp.sh
     chmod a+x mcpp.sh
     if test x$cpp_base = xcc1; then
-        echo $cpp_path/$mcpp_name -+ '"$@"'  >> mcpp_plus.sh
+        echo $cpp_path/$mcpp_name -+ '"$@"' >> mcpp_plus.sh
         chmod a+x mcpp_plus.sh
     fi
 fi
@@ -190,6 +199,7 @@ else
     echo "  $LN_S mcpp.sh $cpp_name"
     $LN_S mcpp.sh $cpp_name
 fi
+
 if test x$cpp_base = xcc1; then
     if test $host_system = SYS_MINGW; then
         echo "  cp cc1$EXEEXT cc1plus$EXEEXT"
