@@ -6,7 +6,11 @@
 
 main( void)
 {
-    char *  cp = str( "p");  /* 0xe0a170 */
+    char *  cp = str( "字");   /* 0xe5ad97 : legal */
+    char *  ecp1 = str( "");   /* 0xc0af   : overlong  */
+    char *  ecp2 = str( "");   /* 0xe09fbf : overlong  */
+    char *  ecp3 = str( "");   /* 0xeda080 : UTF-16 surrogate  */
+
     return  0;
 }
 
